@@ -143,7 +143,7 @@ let done_files =
   
 let files = 
   define_option files_section ["files"] 
-    "The files currently being downloaded" (
+    "The files currently being downloaded, primary downloads must come first" (
     listiter_option (FileOption.t false)) []
 
 (*************************************************************************)
@@ -1155,3 +1155,9 @@ let max_filedescs = (max_all_sockets - max_sockets) / 2 *)
   Unix32.max_cache_size := total_files - 20;
   calc_real_max_indirect_connections ()
 )
+
+let _ =
+  Heap.add_memstat "CommonComplexOptions" (fun level buf ->
+      Printf.bprintf buf "  friends: %d\n" (List.length !!friends);
+      Printf.bprintf buf "  contacts: %d\n" (List.length !contacts);
+  )
