@@ -359,9 +359,6 @@ let file_completed (file : file) =
         files =:= List2.removeq file !!files;
         done_files =:= file :: !!done_files;
         update_file_state impl FileDownloaded;
-        let file_name = file_disk_name file in
-        ignore (CommonShared.new_shared "completed" 0 (
-            file_best_name file ) file_name);
         (try mail_for_completed_file file with e ->
               lprintf_nl "Exception %s in sendmail" (Printexc2.to_string e);
               );
@@ -426,7 +423,8 @@ let friend_add c =
       client_must_update c;
       friends =:= c :: !!friends;
       contacts := List2.removeq c !contacts;
-      impl.impl_client_ops.op_client_browse impl.impl_client_val true
+      if network_is_enabled ((as_client_impl c).impl_client_ops.op_client_network) then
+        impl.impl_client_ops.op_client_browse impl.impl_client_val true
     end
 
 (* Maybe we should not add the client to the contact list and completely remove
@@ -456,7 +454,8 @@ let contact_add c =
       set_contact c;
       client_must_update c;
       contacts := c :: !contacts;
-      impl.impl_client_ops.op_client_browse impl.impl_client_val true
+      if network_is_enabled ((as_client_impl c).impl_client_ops.op_client_network) then
+        impl.impl_client_ops.op_client_browse impl.impl_client_val true
     end
 
 let contact_remove c =
