@@ -455,6 +455,7 @@ let verbosity = define_expert_option current_section ["verbosity"]
   sm : debug source management
   net : debug net
   gui : debug gui
+  no-login : disable login messages
   file : debug file handling
   do : some download warnings
   up : some upload warnings
@@ -487,7 +488,7 @@ let current_section = bandwidth_section
 let max_hard_upload_rate = define_option current_section ["max_hard_upload_rate"]
   "The maximal upload rate you can tolerate on your link in kBytes/s (0 = no limit)
   The limit will apply on all your connections (clients and servers) and both
-control and data messages." int_option 7
+control and data messages." int_option 10
 
 let max_hard_download_rate = define_option current_section ["max_hard_download_rate"]
   "The maximal download rate you can tolerate on your link in kBytes/s (0 = no limit)
@@ -501,7 +502,7 @@ let max_opened_connections = define_option current_section
 let max_indirect_connections = define_option current_section
     ["max_indirect_connections"]
   "Amount of indirect connections in percent (min 30, max 70) of max_opened_connections"
-  int_option 20
+  int_option 30
 
 let max_upload_slots = define_option current_section ["max_upload_slots"]
     "How many slots can be used for upload"
@@ -511,6 +512,10 @@ let friends_upload_slot = define_option current_section ["friends_upload_slot"]
     "Set aside a single reserved slot to upload to friends"
     bool_option true
 
+let small_files_slot_limit = define_option current_section ["small_files_slot_limit"]
+  "Maximum file size to benefit from the reserved slot for small files (0 to disable)"
+    int64_option 10240L
+
 let dynamic_slots = define_option current_section ["dynamic_slots"]
     "Set this to true if you want to have dynamic upload slot allocation (experimental)" bool_option false
 
@@ -552,17 +557,20 @@ let current_section = networks_section
 
 let enable_overnet = define_option current_section
     ["enable_overnet"]
-  "Set to true if you also want mldonkey to run as an overnet client (enable_donkey must be true)"
+  "Set to true if you also want mldonkey to run as an overnet client
+   (enable_donkey must be true)"
     bool_option false
 
 let enable_kademlia = define_option current_section
     ["enable_kademlia"]
-  "Set to true if you also want mldonkey to run as an kademlia client (enable_donkey must be true, and only experimental)"
+  "Set to true if you also want mldonkey to run as an kademlia client
+   (enable_donkey must be true, and only experimental)"
     bool_option false
 
 let enable_servers = define_option current_section
     ["enable_servers"]
-  "Set to true if you also want mldonkey to run with edonkey servers (enable_donkey must be true, and only experimental)"
+  "Set to true if you want mldonkey to connect to edonkey servers
+   (enable_donkey must be true, and only experimental)"
     bool_option true
 
 let enable_bittorrent = define_option current_section
@@ -776,7 +784,8 @@ let html_mods_bw_refresh_delay = define_option current_section
     ["html_mods_bw_refresh_delay"] "bw_stats refresh delay (seconds)" int_option 11
 
 let html_mods_theme = define_option current_section
-    ["html_mods_theme"] "html_mods_theme to use (located in relative html_themes/<theme_name> directory, leave blank to use internal theme"
+    ["html_mods_theme"] "html_mods_theme to use (located in relative html_themes/<theme_name> directory
+     leave blank to use internal theme"
     string_option ""
 
 let use_html_mods o =
@@ -799,6 +808,9 @@ let html_mods_js_tooltips_wait = define_expert_option current_section
 
 let html_mods_js_tooltips_timeout = define_expert_option current_section
     ["html_mods_js_tooltips_timeout"] "How long to display the tooltips" int_option 100000
+    
+let html_mods_use_js_helptext = define_expert_option current_section
+    ["html_mods_use_js_helptext"] "Use javascript to display option help text as js popup (true=use js, false=use html tables)" bool_option true
 
 
 (*************************************************************************)
@@ -988,12 +1000,14 @@ let current_section = download_section
 
 let auto_commit = define_option current_section
     ["auto_commit"]
-  "Set to false if you don't want mldonkey to automatically put completed files in incoming directory"
+  "Set to false if you don't want mldonkey to automatically put completed files
+   in incoming directory"
     bool_option true
 
 let pause_new_downloads = define_option current_section
     ["pause_new_downloads"]
-    "Set to true if you want all new downloads be paused immediatly, will be set to false on core start."
+    "Set to true if you want all new downloads be paused immediatly
+     will be set to false on core start."
     bool_option false
 
   (*
@@ -1020,6 +1034,11 @@ let sources_per_chunk =
     "How many sources to use to download each chunk"
     int_option 3
 
+let swarming_block_selection_algorithm =
+  define_expert_option current_section ["swarming_block_selection_algorithm"]
+    "What algorithm to use to select blocks (currently 1 or 2)"
+    int_option 1
+
   (*
 let delete_original = define_option current_section ["delete_original"]
   "Should MLdonkey delete the file downloaded when splitting has been succesful"
@@ -1356,10 +1375,6 @@ let html_frame_border = define_expert_option current_section ["html_frame_border
 
 let commands_frame_height = define_expert_option current_section ["commands_frame_height"] "The height of the command frame in pixel (depends on your screen and browser sizes)" int_option 46
 
-let motd_html = define_expert_option current_section ["motd_html"]
-    "Message printed at startup (automatically downloaded from the previous
-    URL directory" string_option "<br><div align=\"center\"><h3>Welcome to MLDonkey</h3></div>"
-
 let compaction_delay = define_expert_option current_section ["compaction_delay"]
     "Force compaction every <n> hours (in [1..24])"
     int_option 2
@@ -1543,6 +1558,9 @@ let log_file = define_expert_option current_section ["log_file"]
     also enable logging in a file after startup using the 'log_file' command."
   string_option "mlnet.log"
 
+let gui_log_size =
+  define_expert_option current_section ["gui_log_size"]
+    "number of lines for GUI console messages" int_option 30
 
 
 
@@ -1658,6 +1676,11 @@ let _ =
           close_log ()
         end
   );
+  option_hook swarming_block_selection_algorithm (fun _ ->
+    match !!swarming_block_selection_algorithm with
+    | 1 | 2 -> ()
+    | _ -> swarming_block_selection_algorithm =:= 1;
+  );
   option_hook max_upload_slots (fun _ ->
       if !!max_upload_slots < 3 then
         max_upload_slots =:= 3);
@@ -1730,6 +1753,7 @@ let verbose_msg_servers = ref false
 let verbose = ref false
 let verbose_sources = ref 0
 let verbose_download = ref false
+let verbose_no_login = ref false
 let verbose_upload = ref false
 let verbose_unknown_messages = ref false
 let verbose_overnet = ref false
@@ -1754,8 +1778,10 @@ let set_all v =
   TcpServerSocket.debug := v;
   UdpSocket.debug := v;
   Unix32.verbose := v;
+  GuiProto.verbose_gui_decoding := v;
   verbose_download := v;
   verbose_upload := v;
+  verbose_no_login := v;
   verbose_unknown_messages := v;
   verbose_overnet := v;
   verbose_location := v;
@@ -1786,6 +1812,7 @@ let _ =
           | "net" -> BasicSocket.debug := true; TcpServerSocket.debug := true; UdpSocket.debug := true
           | "file" -> Unix32.verbose := true
           | "gui" -> GuiProto.verbose_gui_decoding := true
+          | "no-login" -> verbose_no_login := true
           | "do" -> verbose_download := true
           | "up" -> verbose_upload := true
           | "unk" -> verbose_unknown_messages := true
