@@ -401,6 +401,7 @@ type result_info = {
 
     mutable result_time : int;
     mutable result_modified : bool;
+    mutable result_source_network : int;
   }
 
 type result = {
@@ -878,3 +879,8 @@ let dummy_stats =
     brand_download = 0L;
     brand_upload = 0L;
   }
+
+type kind_type = {
+  f : string -> string -> unit;
+  description : string
+}
