@@ -595,7 +595,10 @@ let get_file proto s pos =
     file_all_sources = file_all_sources;
     file_active_sources = file_active_sources;
     file_state = state;
-    file_chunks = chunks;
+    file_chunks = 
+      if chunks <> "" then 
+	Some (VerificationBitmap.of_string chunks)
+      else None;
     file_availability = availability;
     file_download_rate = rate;
     file_chunks_age = chunks_age;
@@ -996,7 +999,7 @@ let from_gui (proto : int array) opcode s =
     | 5
     | 52 ->
         if proto < 14 then
-           let pass,_ = get_string s 2 in Password (CommonUserDb.admin_user, pass)
+           let pass = fst (get_string s 2) in Password (CommonUserDb.admin_user, pass)
         else
         let pass,pos = get_string s 2 in
         let login,pos = get_string s pos in
