@@ -315,7 +315,7 @@ Use '$rhelp command$n' or '$r? command$n' for help on a command.
         if valid_password user pass then begin
             auth := true;
             o.conn_user <- find_ui_user user;
-            lprintf_nl "Authenticated user: %s" user;
+            if not !verbose_no_login then lprintf_nl "Authenticated user: %s" user;
             let module M = CommonMessages in
             Buffer.add_string buf M.full_access
           end else
@@ -571,7 +571,7 @@ let telnet_handler t event =
   match event with
     TcpServerSocket.CONNECTION (s, Unix.ADDR_INET (from_ip, from_port)) ->
       let from_ip = Ip.of_inet_addr from_ip in
-        lprintf_nl "Telnet connection from %s" (Ip.to_string from_ip);
+        if not !verbose_no_login then lprintf_nl "Telnet connection from %s" (Ip.to_string from_ip);
         let token = create_token unlimited_connection_manager in
         let sock = TcpBufferedSocket.create_simple token
           "telnet connection"
@@ -603,7 +603,9 @@ let telnet_handler t event =
         before_telnet_output o sock;
         TcpBufferedSocket.write_string sock
 	   (Printf.sprintf "Welcome to MLDonkey %s\n" Autoconf.current_version);
-        TcpBufferedSocket.write_string sock (DriverInteractive.real_startup_message ());
+	(match DriverInteractive.real_startup_message () with
+	   Some s -> TcpBufferedSocket.write_string sock (s);
+	 | None -> ());
 
         TcpBufferedSocket.write_string sock (dollar_escape o false
             "$cWelcome on mldonkey command-line$n\n\nUse $r?$n for help\n\n");
@@ -618,7 +620,7 @@ let telnet_handler t event =
 	in
 	TcpBufferedSocket.write_string sock (dollar_escape o false reject_message);
 	shutdown sock Closed_connect_failed;
-  lprintf_n "%s" reject_message;
+        if not !verbose_no_login then lprintf_n "%s" reject_message;
 	Unix.close s
       end
 
@@ -1155,14 +1157,10 @@ let http_handler o t r =
 
         | "oneframe.html" ->
             html_open_page buf t r true;
-            Buffer.add_string buf !!motd_html;
-	    Buffer.add_string buf "<p><pre><b><h3>";
-            Buffer.add_string buf (DriverInteractive.real_startup_message ());
-	    Buffer.add_string buf "</b></h3>";
-	    ignore (buildinfo false buf);
-	    Buffer.add_string buf "\n";
-	    ignore (runinfo false buf o);
-	    Buffer.add_string buf "</pre>\n"
+            Buffer.add_string buf (Printf.sprintf "<br><div align=\"center\"><h3>Welcome to MLDonkey %s</h3></div>" Autoconf.current_version);
+	    (match DriverInteractive.real_startup_message () with
+	       Some s -> Buffer.add_string buf (Printf.sprintf "<p><pre><b><h3>%s</b></h3></pre>" s);
+	     | None -> ())
 
         | "bw_updown.png" ->
             (match http_error_no_gd "png" with
