@@ -68,19 +68,18 @@ let do_daily () =
 
 let minute_timer () =
   DriverInteractive.hdd_check ();
-  DriverInteractive.file_magic_check ();
   CommonShared.shared_check_files ();
   CommonUploads.upload_credit_timer ();
   CommonInteractive.force_download_quotas ();
   CommonResult.dummy_result.result_time <- last_time ();
   (try
-      CommonSwarming.verify_some_chunks ()
-    with _ -> ());
+    CommonSwarming.verify_some_chunks ()
+  with _ -> ());
   CommonClient.clear_upload_slots ()
 
 let hourly_timer timer =
   incr CommonWeb.hours;
-  CommonWeb.load_web_infos false;
+  CommonWeb.load_web_infos false false;
   if !CommonWeb.hours mod !!compaction_delay = 0 then Gc.compact ();
   if !!backup_options_delay <> 0
      && !CommonWeb.hours mod !!backup_options_delay = 0 then
@@ -140,39 +139,6 @@ let start_interfaces () =
   add_infinite_timer 1. second_timer
 
 
-let _ =
-  CommonWeb.add_web_kind "motd.html" "Information of the day in HTML format" 
-    (fun _ filename ->
-      lprintf_nl (_b "motd.html changed");
-    motd_html =:= File.to_string filename
-  );
-  CommonWeb.add_web_kind "motd.conf" "Setup changes of the day" 
-    (fun _ filename ->
-    try
-      Unix2.tryopen_read filename (fun ic ->
-	try
-	  while true do
-	    let line = input_line ic in
-	    let cmd, args = String2.cut_at line ' ' in
-	    let name, value = String2.cut_at args ' ' in
-	    match cmd with
-	      | "set" ->
-		  CommonInteractive.set_fully_qualified_options name value
-	      | "add_item" ->
-		  CommonInteractive.add_item_to_fully_qualified_options name value
-	      | "del_item" ->
-		  CommonInteractive.del_item_from_fully_qualified_options name value
-	      | _ ->
-      lprintf_nl (_b "UNUSED LINE: %s") line
-	  done
-	with End_of_file -> ())
-    with
-	e -> 
-    lprintf_nl (_b "Error while reading motd.conf(%s): %s") filename
-	    (Printexc2.to_string e)
-  )
-
-
 let save_mlsubmit_reg () =
 
 (* Generate the mlsubmit.reg file *)
@@ -436,7 +402,6 @@ or getting a binary compiled with glibc %s.\n\n")
 
 (*  lprintf "(1) CommonComplexOptions.load\n"; *)
   CommonComplexOptions.load ();
-  DriverInteractive.file_magic_check ();
   CommonUploads.load ();
 
 (*  lprintf "(2) CommonComplexOptions.load done\n"; *)
@@ -500,7 +465,7 @@ or getting a binary compiled with glibc %s.\n\n")
   Options.prune_file downloads_ini;
   Options.prune_file users_ini;
 (*  Options.prune_file downloads_expert_ini; *)
-  add_timer 5. (fun _ -> try CommonWeb.load_web_infos true with _ -> ());
+  add_timer 5. (fun _ -> try CommonWeb.load_web_infos true false with _ -> ());
   lprintf_nl  (_b "To command: telnet %s %d")
 	(if !!telnet_bind_addr = Ip.any then "127.0.0.1"
 		else Ip.to_string !!telnet_bind_addr)  !!telnet_port;
