@@ -22,6 +22,7 @@
 
 module O = GuiOptions
 module M = GuiMessages
+module VB = VerificationBitmap
 
 let (!!) = Options.(!!)
 let (=:=) = Options.(=:=)
@@ -349,60 +350,65 @@ let clean_icons () =
 (*                                                                                *)
 (**********************************************************************************)
 
-let (avail_bars : ((string * string * bool), GdkPixbuf.pixbuf) Hashtbl.t) = Hashtbl.create 103
+let (avail_bars : ((string * VB.t option * bool), GdkPixbuf.pixbuf) Hashtbl.t) = Hashtbl.create 103
 
 (* make a simple function to give a 3D effect *)
 let highlight range i =
   if i < 8
-    then 256 * (2 * i * range / 16)
-    else 256 * (range - (i * range / 2 / 16))
+    then (2 * i * range / 16)
+    else (range - (i * range / 2 / 16))
 
 (* define the colors we will use to display the availability *)
-let color_red = GDraw.pixmap ~width:1 ~height:16
-                  ~colormap:(Gdk.Color.get_system_colormap ()) ()
+let color_red = GdkPixbuf.create ~width:1 ~height:16 ~has_alpha:true ()
 let _ =
+  let pixb = GdkPixbuf.create ~width:1 ~height:1 ~has_alpha:true () in
   for i = 0 to 15 do
     let r = highlight 255 i in
-    color_red#set_foreground (`RGB (r, 0, 0));
-    color_red#point ~x:0 ~y:i
+    let s =  Printf.sprintf "0x%02X0000ff" r in
+    GdkPixbuf.fill pixb (Int32.of_string s);
+    GdkPixbuf.copy_area ~dest:color_red ~dest_x:0 ~dest_y:i ~width:1 ~height:1 ~src_x:0 ~src_y:0 pixb
   done
-    
-let color_green = GDraw.pixmap ~width:1 ~height:16
-                    ~colormap:(Gdk.Color.get_system_colormap ()) ()
+
+let color_green = GdkPixbuf.create ~width:1 ~height:16 ~has_alpha:true ()
 let _ =
+  let pixb = GdkPixbuf.create ~width:1 ~height:1 ~has_alpha:true () in
   for i = 0 to 15 do
-    let g = highlight 255 i in
-    color_green#set_foreground (`RGB (0, g, 0));
-    color_green#point ~x:0 ~y:i
+    let r = highlight 255 i in
+    let s =  Printf.sprintf "0x00%02X00ff" r in
+    GdkPixbuf.fill pixb (Int32.of_string s);
+    GdkPixbuf.copy_area ~dest:color_green ~dest_x:0 ~dest_y:i ~width:1 ~height:1 ~src_x:0 ~src_y:0 pixb
   done
-    
-let color_black = GDraw.pixmap ~width:1 ~height:16
-                    ~colormap:(Gdk.Color.get_system_colormap ()) ()
+
+let color_black = GdkPixbuf.create ~width:1 ~height:16 ~has_alpha:true ()
 let _ =
+  let pixb = GdkPixbuf.create ~width:1 ~height:1 ~has_alpha:true () in
   for i = 0 to 15 do
     let r = highlight 128 i in
-    color_black#set_foreground (`RGB (r, r, r));
-    color_black#point ~x:0 ~y:i
+    let s =  Printf.sprintf "0x%02X%02X%02Xff" r r r in
+    GdkPixbuf.fill pixb (Int32.of_string s);
+    GdkPixbuf.copy_area ~dest:color_black ~dest_x:0 ~dest_y:i ~width:1 ~height:1 ~src_x:0 ~src_y:0 pixb
   done
-    
-let color_orange = GDraw.pixmap ~width:1 ~height:16
-                     ~colormap:(Gdk.Color.get_system_colormap ()) ()
+
+let color_orange = GdkPixbuf.create ~width:1 ~height:16 ~has_alpha:true ()
 let _ =
+  let pixb = GdkPixbuf.create ~width:1 ~height:1 ~has_alpha:true () in
   for i = 0 to 15 do
     let r = highlight 255 i in
     let g = 178 * r / 255 in
-    color_orange#set_foreground (`RGB (r, g, 0));
-    color_orange#point ~x:0 ~y:i
+    let s =  Printf.sprintf "0x%02X%02X00ff" r g in
+    GdkPixbuf.fill pixb (Int32.of_string s);
+    GdkPixbuf.copy_area ~dest:color_orange ~dest_x:0 ~dest_y:i ~width:1 ~height:1 ~src_x:0 ~src_y:0 pixb
   done
 
-let color_yellow = GDraw.pixmap ~width:1 ~height:16
-                     ~colormap:(Gdk.Color.get_system_colormap ()) ()
+let color_yellow = GdkPixbuf.create ~width:1 ~height:16 ~has_alpha:true ()
 let _ =
+  let pixb = GdkPixbuf.create ~width:1 ~height:1 ~has_alpha:true () in
   for i = 0 to 15 do
-    let r = highlight 255 i in
+     let r = highlight 255 i in
     let g = 255 * r / 255 in
-    color_yellow#set_foreground (`RGB (r, g, 0));
-    color_yellow#point ~x:0 ~y:i
+    let s =  Printf.sprintf "0x%02X%02X00ff" r g in
+    GdkPixbuf.fill pixb (Int32.of_string s);
+    GdkPixbuf.copy_area ~dest:color_yellow ~dest_x:0 ~dest_y:i ~width:1 ~height:1 ~src_x:0 ~src_y:0 pixb
   done
 
 let color_blue_relative = ref [||]
@@ -410,27 +416,29 @@ let color_blue_relative = ref [||]
 let create_color_blue_relative () =
   color_blue_relative := [||];
   for i = 0 to (!!O.gtk_misc_availability_max - 1) do
-    let pixmap = GDraw.pixmap ~width:1 ~height:16
-                   ~colormap:(Gdk.Color.get_system_colormap ()) () in
+    let pixbuf = GdkPixbuf.create ~width:1 ~height:16 ~has_alpha:true () in
     let col_step = i * 255 / (!!O.gtk_misc_availability_max - 1) in
+    let pixb = GdkPixbuf.create ~width:1 ~height:1 ~has_alpha:true () in
     for j = 0 to 15 do
       let b = highlight 255 j in
       let g = highlight col_step j in
-      pixmap#set_foreground (`RGB (0, g, b));
-      pixmap#point ~x:0 ~y:j
+      let s =  Printf.sprintf "0x00%02X%02Xff" g b in
+      GdkPixbuf.fill pixb (Int32.of_string s);
+      GdkPixbuf.copy_area ~dest:pixbuf ~dest_x:0 ~dest_y:j ~width:1 ~height:1 ~src_x:0 ~src_y:0 pixb
     done;
-    color_blue_relative := Array.append !color_blue_relative [|pixmap|]
+    color_blue_relative := Array.append !color_blue_relative [|pixbuf|]
   done
 
 let _ = create_color_blue_relative ()
 
-let color_grey = GDraw.pixmap ~width:1 ~height:16
-                   ~colormap:(Gdk.Color.get_system_colormap ()) ()
+let color_grey = GdkPixbuf.create ~width:1 ~height:16 ~has_alpha:true ()
 let _ =
+  let pixb = GdkPixbuf.create ~width:1 ~height:1 ~has_alpha:true () in
   for i = 0 to 15 do
     let r = highlight 255 i in
-    color_grey#set_foreground (`RGB (r, r, r));
-    color_grey#point ~x:0 ~y:i
+    let s =  Printf.sprintf "0x%02X%02X%02Xff" r r r in
+    GdkPixbuf.fill pixb (Int32.of_string s);
+    GdkPixbuf.copy_area ~dest:color_grey ~dest_x:0 ~dest_y:i ~width:1 ~height:1 ~src_x:0 ~src_y:0 pixb
   done
 
 let normalize_availability avail =
@@ -439,13 +447,17 @@ let normalize_availability avail =
     if !!O.gtk_misc_use_availability_height
       then if (int_of_char avail.[i]) > !!O.gtk_misc_availability_max
         then avail.[i] <- (char_of_int !!O.gtk_misc_availability_max)
+        else ()
       else if (int_of_char avail.[i]) > 1
         then avail.[i] <- (char_of_int 1)
+        else ()
   done
 
 let get_availability_of availability chunks is_file =
   let height = 16 in
-  let nchunks = max 1 (String.length chunks) in
+  let nchunks = match chunks with
+    | None -> 1
+    | Some chunks -> max 1 (VB.length chunks) in
   let avail =
     if is_file
       then begin
@@ -457,74 +469,59 @@ let get_availability_of availability chunks is_file =
   let key = (avail, chunks, is_file) in
   try
     Hashtbl.find avail_bars key
-  with _ ->
+  with Not_found ->
     begin
       (if !!verbose then lprintf' "Creating new availability bar\n");
-      let pixmap = GDraw.pixmap ~width:nchunks ~height
-        ~colormap:(Gdk.Color.get_system_colormap ()) ()
-      in
+      let dest = GdkPixbuf.create ~width:nchunks ~height ~has_alpha:true () in
       (try
+	match chunks with
+	| None -> ()
+	| Some chunks ->
         for i = 0 to (nchunks - 1) do
           if is_file
-            then if chunks.[i] >= '2'
-              then pixmap#put_pixmap
-                     ~x:i ~y:0 ~xsrc:0 ~ysrc:0 ~width:1 ~height:height
-                     color_green#pixmap
-              else begin
+          then 
+	    (match VB.get chunks i with
+	    | VB.State_complete | VB.State_verified ->
+		GdkPixbuf.copy_area ~dest ~dest_x:i ~dest_y:0 ~width:1 ~height ~src_x:0 ~src_y:0 color_green
+	    | VB.State_missing | VB.State_partial -> 
                 let h = int_of_char (avail.[i]) in
-                if h = 0
-                  then if chunks.[i] = '0'
-                    then pixmap#put_pixmap
-                           ~x:i ~y:0 ~xsrc:0 ~ysrc:0 ~width:1 ~height:height
-                           color_red#pixmap
-                    else pixmap#put_pixmap
-                           ~x:i ~y:0 ~xsrc:0 ~ysrc:0 ~width:1 ~height:height
-                           color_orange#pixmap
-                  else if chunks.[i] = '0'
-                    then begin
-                      let color_blue = !color_blue_relative.(!!O.gtk_misc_availability_max - h) in
-                      pixmap#put_pixmap
-                        ~x:i ~y:0 ~xsrc:0 ~ysrc:0 ~width:1 ~height:height
-                        color_blue#pixmap
-                    end else pixmap#put_pixmap
-                               ~x:i ~y:0 ~xsrc:0 ~ysrc:0 ~width:1 ~height:height
-                               color_yellow#pixmap
-              end
-            else if int_of_char avail.[i] >= 49
-              then if chunks.[i] >= '2'
-                then pixmap#put_pixmap
-                         ~x:i ~y:0 ~xsrc:0 ~ysrc:0 ~width:1 ~height:height
-                         color_black#pixmap
-                else if chunks.[i] = '0'
-                  then pixmap#put_pixmap
-                           ~x:i ~y:0 ~xsrc:0 ~ysrc:0 ~width:1 ~height:height
-                           color_green#pixmap
-                  else pixmap#put_pixmap
-                           ~x:i ~y:0 ~xsrc:0 ~ysrc:0 ~width:1 ~height:height
-                           color_yellow#pixmap
-              else pixmap#put_pixmap
-                       ~x:i ~y:0 ~xsrc:0 ~ysrc:0 ~width:1 ~height:height
-                       color_red#pixmap
-
-          done;
+		match h, VB.get chunks i with
+		| 0, VB.State_missing ->
+		    GdkPixbuf.copy_area ~dest ~dest_x:i ~dest_y:0 ~width:1 ~height ~src_x:0 ~src_y:0 color_red
+		| 0, _ ->
+		    GdkPixbuf.copy_area ~dest ~dest_x:i ~dest_y:0 ~width:1 ~height ~src_x:0 ~src_y:0 color_orange
+		| _, VB.State_missing ->
+                    let color_blue = !color_blue_relative.(!!O.gtk_misc_availability_max - h) in
+                    GdkPixbuf.copy_area ~dest ~dest_x:i ~dest_y:0 ~width:1 ~height ~src_x:0 ~src_y:0 color_blue
+		| _, _ ->
+		    GdkPixbuf.copy_area ~dest ~dest_x:i ~dest_y:0 ~width:1 ~height ~src_x:0 ~src_y:0 color_yellow
+            )
+          else 
+	    match int_of_char avail.[i] >= 49, VB.get chunks i with
+	    | true, VB.State_complete
+	    | true, VB.State_verified ->
+		GdkPixbuf.copy_area ~dest ~dest_x:i ~dest_y:0 ~width:1 ~height ~src_x:0 ~src_y:0 color_black
+	    | true, VB.State_missing ->
+		GdkPixbuf.copy_area ~dest ~dest_x:i ~dest_y:0 ~width:1 ~height ~src_x:0 ~src_y:0 color_green
+	    | true, VB.State_partial ->
+		GdkPixbuf.copy_area ~dest ~dest_x:i ~dest_y:0 ~width:1 ~height ~src_x:0 ~src_y:0 color_yellow
+	    | false, _ ->
+		GdkPixbuf.copy_area ~dest ~dest_x:i ~dest_y:0 ~width:1 ~height ~src_x:0 ~src_y:0 color_red
+        done;
 
       with _ ->
-        begin
-          (if !!verbose then lprintf' "Exception in creating avail_bar avail: %s chunks: %s file: %b\n" avail chunks is_file);
-          for i = 0 to nchunks do
-            pixmap#put_pixmap
-              ~x:i ~y:0 ~xsrc:0 ~ysrc:0 ~width:1 ~height:height
-              color_grey#pixmap
-          done
-        end);
-    let pb = GdkPixbuf.create ~width:nchunks ~height ~has_alpha:false () in
-    GdkPixbuf.get_from_drawable 
-       ~dest:pb ~dest_x:0 ~dest_y:0 
-       ~width:nchunks ~height ~src_x:0 ~src_y:0
-       ~colormap:(Gdk.Color.get_system_colormap ()) 
-       pixmap#pixmap;
-    Hashtbl.add avail_bars key pb;
-    pb
+          if !!verbose then 
+	    lprintf' "Exception in creating avail_bar avail: %s chunks: %s file: %b\n" 
+	      avail 
+	      (match chunks with
+	      |	None -> ""
+	      | Some chunks -> VB.to_string chunks) 
+	      is_file;
+          for i = 0 to nchunks - 1 do
+            GdkPixbuf.copy_area ~dest ~dest_x:i ~dest_y:0 ~width:1 ~height ~src_x:0 ~src_y:0 color_grey
+          done);
+    Hashtbl.add avail_bars key dest;
+    dest
   end
 
 
@@ -556,117 +553,3 @@ let clean_avail_bars list =
              lprintf' "   -----------------------\n   avail_bars : %d bars\n"  (List.length l)
            end
        end
-
-
-
-(*
-doesn't work ! pixbuf are freed as soon as they are rendered !!!!
-
-open Gui_types
-
-let dummy_bar =
-  {
-   finder = ("", "", true);
-   pixbuf = GdkPixbuf.create ~width:1 ~height:1 ();
-  }
-
-module H = Weak2.Make(struct
-  type t = avail_bar
-  let hash s = Hashtbl.hash s.finder
-            
-  let equal x y = x.finder = y.finder
-end)
-
-let avail_bars = H.create 1023
-
-let f = (fun _ ->
-  Printf.printf "Finalising availability bar\n";
-  flush stdout
-  )
-
-let get_availability_of availability chunks is_file =
-  let height = 16 in
-  let nchunks = String.length chunks in
-  let avail =
-    if is_file
-      then begin
-        let s = String.copy availability in
-        normalize_availability s;
-        s
-      end else availability
-  in
-  let key = (avail, chunks, is_file) in
-  try
-    Printf.printf "Trying to use an existing availability bar\n";
-    flush stdout;
-    let bar = {dummy_bar with finder = (avail, chunks, is_file)} in
-    let pixb = H.find avail_bars bar in
-    pixb
-  with _ ->
-    begin
-      Printf.printf "Creating new availability bar\n";
-      flush stdout;
-      let pixmap = GDraw.pixmap ~width:nchunks ~height
-        ~colormap:(Gdk.Color.get_system_colormap ()) ()
-      in
-      (try
-        for i = 0 to (nchunks - 1) do
-          if is_file
-            then if chunks.[i] >= '2'
-              then pixmap#put_pixmap
-                     ~x:i ~y:0 ~xsrc:0 ~ysrc:0 ~width:1 ~height:height
-                     color_green#pixmap
-              else begin
-                let h = int_of_char (avail.[i]) in
-                if h = 0
-                  then if chunks.[i] = '0'
-                    then pixmap#put_pixmap
-                           ~x:i ~y:0 ~xsrc:0 ~ysrc:0 ~width:1 ~height:height
-                           color_red#pixmap
-                    else pixmap#put_pixmap
-                           ~x:i ~y:0 ~xsrc:0 ~ysrc:0 ~width:1 ~height:height
-                           color_orange#pixmap
-                  else begin
-                    let color_blue = !color_blue_relative.(!!O.gtk_misc_availability_max - h) in
-                    pixmap#put_pixmap
-                      ~x:i ~y:0 ~xsrc:0 ~ysrc:0 ~width:1 ~height:height
-                      color_blue#pixmap
-                  end
-              end
-            else if int_of_char avail.[i] >= 49
-              then if chunks.[i] >= '2'
-                then pixmap#put_pixmap
-                       ~x:i ~y:0 ~xsrc:0 ~ysrc:0 ~width:1 ~height:height
-                       color_black#pixmap
-                else pixmap#put_pixmap
-                       ~x:i ~y:0 ~xsrc:0 ~ysrc:0 ~width:1 ~height:height
-                       color_green#pixmap
-              else if chunks.[i] > '2'
-                then pixmap#put_pixmap
-                       ~x:i ~y:0 ~xsrc:0 ~ysrc:0 ~width:1 ~height:height
-                       color_orange#pixmap
-                else pixmap#put_pixmap
-                       ~x:i ~y:0 ~xsrc:0 ~ysrc:0 ~width:1 ~height:height
-                       color_red#pixmap
-          done;
-
-      with _ ->
-          begin
-            for i = 0 to nchunks do
-              pixmap#put_pixmap
-                ~x:i ~y:0 ~xsrc:0 ~ysrc:0 ~width:1 ~height:height
-                color_grey#pixmap
-            done
-          end);
-    let pb = GdkPixbuf.create ~width:nchunks ~height ~has_alpha:false () in
-    GdkPixbuf.get_from_drawable 
-       ~dest:pb ~dest_x:0 ~dest_y:0 
-       ~width:nchunks ~height ~src_x:0 ~src_y:0
-       ~colormap:(Gdk.Color.get_system_colormap ()) 
-       pixmap#pixmap;
-    let bar = {dummy_bar with finder = key; pixbuf = pb} in
-    Gc.finalise f bar;
-    H.add avail_bars bar;
-    bar
-  end
-*)
