@@ -23,7 +23,79 @@ open Printf2
 open Options
 open Gettext
 
-let filename = Filename.concat CommonOptions.home_dir "mlnet_strings"
+let bin_dir = Filename.dirname Sys.argv.(0)
+
+let hidden_dir_prefix =
+  if Autoconf.system = "windows" then "" else "."
+
+let config_dir_basename = hidden_dir_prefix ^ "mldonkey"
+
+let home_dir =
+  match Autoconf.system with
+  | "windows" -> if (Filename.basename bin_dir) = "bin" then Filename.dirname bin_dir else "."
+  | _ -> Filename.concat (try Sys.getenv "HOME" with _ -> ".") config_dir_basename
+
+let file_basedir_pre =
+  try
+    let s = Sys.getenv "MLDONKEY_DIR" in
+    if s = "" then home_dir else Filename2.normalize s
+  with _ -> home_dir
+
+let file_basedir =
+(* Creating dirs does work differently on Windows than Unix.
+   Dirs like c:\b are split down by unix2.safe_mkdir to "c".
+   This function splits the directory name into the drive name
+   and chdir to it before creating directories.
+   Non-absolute paths in $MLDONKEY_DIR do not work as well *)
+  if Sys.file_exists (Filename.concat (Sys.getcwd ()) "mlgui.ini") then
+    "."
+  else
+    if Autoconf.system = "windows" && file_basedir_pre <> home_dir then
+      match String2.split file_basedir_pre ':' with
+      | drive :: directory :: _ ->
+          Unix.chdir (drive ^ ":\\");
+          directory
+      | _ -> lprintf "Please provide an absolute path in MLDONKEY_DIR like d:\\mldonkey, exiting...\n"; exit 2
+    else file_basedir_pre
+
+let _ =
+  if not !CommonGlobals.core_included
+    then begin
+      (try
+         Unix2.safe_mkdir file_basedir
+       with e ->
+         lprintf_nl "Exception (%s) trying to create dir %s"
+           (Printexc2.to_string e) file_basedir;
+         exit 2);
+      Unix2.can_write_to_directory file_basedir;
+      Unix.chdir file_basedir;
+      lprintf_nl "Starting MLGui %s ... " Autoconf.current_version;
+      lprintf_nl "MLGui is working in %s" (Sys.getcwd ())
+    end
+
+
+let dirname_concat dir =
+  match Autoconf.system with
+      "windows" -> Filename.concat file_basedir dir
+    | _ -> file_basedir
+
+let lang_dir = dirname_concat "lang"
+let gui_config_dir = dirname_concat "interface"
+let log_dir = dirname_concat "logs"
+
+let _ =
+  if not !CommonGlobals.core_included
+  then begin
+    let filename =
+      try
+        Sys.getenv "MLGUI_STRINGS"
+      with _ ->
+        Filename.concat lang_dir "mlgui_strings"
+    in
+    Unix2.safe_mkdir (Filename.dirname filename);
+    Unix2.can_write_to_directory (Filename.dirname filename);
+    set_strings_file filename
+  end
 
 let _s x = (* lprintf "searching string %s\n" x; *) GuiUtf8.simple_utf8_of (_s "GuiMessages" x)
 
@@ -163,10 +235,18 @@ let dT_me_save_as  = ref ""
 let dT_me_save  = ref ""
 let dT_me_browse_files  = ref ""
 let dT_me_add_to_friends  = ref ""
+let dT_me_razorback2_stats = ref ""
+let dT_me_filedonkey_stats = ref ""
+let dT_me_isohunt_stats = ref ""
 let dT_lb_link  = ref ""
 let dT_me_show_file_details  = ref ""
 let dT_me_show_source_details  = ref ""
 let dT_wt_show_source_details  = ref ""
+let dT_lb_stats_get_main_page  = ref ""
+let dT_lb_stats_get_image  = ref ""
+let dT_lb_stats_rate  = ref ""
+let dT_lb_stats_available  = ref ""
+let dT_lb_stats_complete  = ref ""
 
 (* Friends Tab *)
 
@@ -240,6 +320,9 @@ let qT_lb_size  = ref ""
 let qT_lb_year = ref ""
 let qT_lb_genre = ref ""
 let qT_lb_comment = ref ""
+let qT_lb_advanced_search = ref ""
+let qT_lb_network = ref ""
+let qT_lb_min_availability = ref ""
 
 (* Rooms Tab *)
 
@@ -270,6 +353,18 @@ let cT_lb_command  = ref ""
 
 (* Graph *)
 
+let gT_lb_time_range = ref ""
+let gT_lb_quarter = ref ""
+let gT_lb_hour = ref ""
+let gT_lb_halfday = ref ""
+let gT_lb_day = ref ""
+let gT_lb_week = ref ""
+let gT_lb_month = ref ""
+let gT_lb_year = ref ""
+let gT_lb_global_downloads = ref ""
+let gT_lb_global_uploads = ref ""
+let gT_lb_file_down_up = ref ""
+
 (* file Window *)
 
 let fW_wt_show_file_details  = ref ""
@@ -467,6 +562,7 @@ let iM_lb_accounts = ref ""
 (* Common Menus labels *)
 
 let mAutosize  = ref ""
+let mAutosize_all = ref ""
 let mSort  = ref ""
 let mRemove_column  = ref ""
 let mAdd_column_after  = ref ""
@@ -613,6 +709,14 @@ let load_messages () =
   dT_me_show_file_details  :=  _s "Show file details";
   dT_me_show_source_details  :=  _s "Show source details";
   dT_wt_show_source_details  :=  _s "Source details";
+  dT_me_razorback2_stats := _s "Get/Refresh Razorback Stats";
+  dT_me_filedonkey_stats := _s "Get/Refresh FileDonkey Stats";
+  dT_me_isohunt_stats := _s "Get/Refresh IsoHunt Stats";
+  dT_lb_stats_get_main_page := _s "Getting main page for file";
+  dT_lb_stats_get_image := _s "Getting graph image for file";
+  dT_lb_stats_rate := _s_ "Rate";
+  dT_lb_stats_available := _s_ "Availability";
+  dT_lb_stats_complete := _s_ "Complete";
 
 (* Friends Tab *)
 
@@ -657,7 +761,7 @@ let load_messages () =
   qT_tx_image  :=  _s "Image";
   qT_tx_documentation  :=  _s "Documentation";
   qT_tx_collection  :=  _s "Collection";
-  qT_lb_keywords  :=  _s "Keywords";
+
   qT_lb_media  :=  _s "Media";
   qT_lb_format  :=  _s "Format";
   qT_lb_min_size  :=  _s "Min size";
@@ -685,6 +789,10 @@ let load_messages () =
   qT_lb_year := _s "Year";
   qT_lb_genre := _s "Genre";
   qT_lb_comment := _s "Comment";
+  qT_lb_advanced_search := _s "Advanced search";
+  qT_lb_network := _s_ "Network";
+  qT_lb_keywords  := _s_ "Keywords";
+  qT_lb_min_availability := _s_ "Min Availability";
 
 (* Rooms Tab *)
 
@@ -715,6 +823,18 @@ let load_messages () =
 
 (* Graph *)
 
+  gT_lb_time_range := _s_ "Time range";
+  gT_lb_quarter := _s "Quarter";
+  gT_lb_hour := _s "Hour";
+  gT_lb_halfday := _s "Half day";
+  gT_lb_day := _s "Day";
+  gT_lb_week := _s "Week";
+  gT_lb_month := _s "Month";
+  gT_lb_year := _s "Year";
+  gT_lb_global_downloads := _s "Global Downloads";
+  gT_lb_global_uploads := _s "Global Uploads";
+  gT_lb_file_down_up := _s "Files Uploads and Downloads";
+
 (* file Window *)
 
   fW_wt_show_file_details  :=  _s "File details";
@@ -912,6 +1032,7 @@ let load_messages () =
 (* Common Menus labels *)
 
   mAutosize  :=  _s "Autosize";
+  mAutosize_all  :=  _s "Autosize All Columns";
   mSort  :=  _s "Sort";
   mRemove_column  :=  _s "Remove Column";
   mAdd_column_after  :=  _s "Add Column After";
@@ -969,8 +1090,9 @@ let load_messages () =
 
 (* Messages and string constants. *)
 
-let chat_config_file = 
-  Filename.concat CommonOptions.home_dir "chat.ini"
+let _ = Unix2.safe_mkdir gui_config_dir
+
+let chat_config_file = Filename.concat gui_config_dir "chat.ini"
 
 (* {2 Command line messages} *)
 
