@@ -64,6 +64,8 @@ open BTChooser
 open BTStats
 open TcpMessages
 
+module VB = VerificationBitmap
+
 let http_ok = "HTTP 200 OK"
 let http11_ok = "HTTP/1.1 200 OK"
 
@@ -383,28 +385,31 @@ let send_interested c =
 (** Send a Bitfield message to a client.
   @param c The client to send the Bitfield message
 *)
+
 let send_bitfield c =
-  let bitmap =
-    match c.client_file.file_swarmer with
-      None ->
-(* This must be a seeded file... *)
-        String.make (Array.length c.client_file.file_chunks) '3'
-    | Some swarmer ->
-        CommonSwarming.verified_bitmap swarmer
-  in
-
-  if !verbose_download then lprintf_nl "Sending verified bitmap: [%s]" bitmap;
-
-(* In the future, only accept bitmap.[n] > '2' when verification works *)
   send_client c (BitField
       (
-      let nchunks = String.length bitmap in
-      let len = (nchunks+7)/8 in
-      let s = String.make len '\000' in
-      for i = 0 to nchunks - 1 do
-        if bitmap.[i] >= '2' then set_bit s i;
-      done;
-      s
+      match c.client_file.file_swarmer with
+      | None ->
+	  (* This must be a seeded file... *)
+	  if !verbose_download then 
+	    lprintf_nl "Sending completed verified bitmap";
+	  let nchunks = Array.length c.client_file.file_chunks in
+	  let len = (nchunks+7)/8 in
+	  let s = String.make len '\000' in
+	  for i = 0 to nchunks - 1 do
+	    set_bit s i
+	  done;
+	  s
+      | Some swarmer ->
+	  let bitmap = CommonSwarming.chunks_verified_bitmap swarmer in
+	  if !verbose_download then 
+	    lprintf_nl "Sending verified bitmap: [%s]" (VB.to_string bitmap);
+	  let len = VB.length bitmap in
+	  let s = String.make len '\000' in
+	  VB.iteri (fun i c ->
+            if c = VB.State_verified then set_bit s i) bitmap;
+	  s
     ))
 
 let counter = ref 0
@@ -550,7 +555,7 @@ and update_client_bitmap c =
     match c.client_uploader with
       None ->
         let up = CommonSwarming.register_uploader swarmer (as_client c)
-          (AvailableRanges []) in
+          (AvailableIntervals []) in
         c.client_uploader <- Some up;
         up
     | Some up ->
@@ -570,7 +575,7 @@ and update_client_bitmap c =
     let chunks = c.client_new_chunks in
     c.client_new_chunks <- [];
     List.iter (fun n -> Bitv.set bitmap n true) chunks;
-    CommonSwarming.update_uploader up (AvailableBitv bitmap);
+    CommonSwarming.update_uploader_intervals up (AvailableBitv bitmap);
   end
 
 
@@ -680,7 +685,7 @@ and get_from_client sock (c: client) =
                         c.client_range_waiting <- None;
                         (x,y,r)
                   | None -> 
-                        CommonSwarming.find_range up 
+                        CommonSwarming.find_range up (min max_range_len file.file_piece_size)
                 in
 
                 let (x,y,r) =
@@ -695,7 +700,7 @@ and get_from_client sock (c: client) =
                   c.client_ranges_sent <- c.client_ranges_sent @ [x,y, r];
 (*                CommonSwarming.alloc_range r; *)
                 
-                  let num = CommonSwarming.block_num swarmer b in
+                  let num = CommonSwarming.block_chunk_num swarmer b in
 
                   if !verbose_swarming then 
                     lprintf_nl "Asking %d For Range %Ld-%Ld" num x y;
@@ -906,13 +911,15 @@ and client_to_client c sock msg =
                 disconnect_client c (Closed_for_error "Wrong bitfield length")
               end else begin
 
-                let verified = CommonSwarming.verified_bitmap swarmer in
+                let bitmap = CommonSwarming.chunks_verified_bitmap swarmer in
 
                 for i = 0 to npieces - 1 do
                   if is_bit_set p i then begin
                     c.client_new_chunks <- i :: c.client_new_chunks;
-                    if verified.[i] < '2' then
-                      c.client_interesting <- true;
+                    match VB.get bitmap i with
+		    | VB.State_missing | VB.State_partial ->
+                      c.client_interesting <- true
+		    | VB.State_complete | VB.State_verified -> ()
                   end 
                 done;
 
@@ -941,15 +948,16 @@ and client_to_client c sock msg =
             None -> ()
           | Some swarmer ->
               let n = Int64.to_int n in
-              let verified = CommonSwarming.verified_bitmap swarmer in
-              (* lprintf_nl "verified: %c;" verified.[n]; *)
+              let bitmap = CommonSwarming.chunks_verified_bitmap swarmer in
+              (* lprintf_nl "verified: %c;" (VB.state_to_char (VB.get bitmap n)); *)
               (* if the peer has a chunk we don't, tell him we're interested and update his bitmap *)
-              if verified.[n] < '2' then begin
+	      match VB.get bitmap n with
+	      | VB.State_missing | VB.State_partial ->
                   c.client_interesting <- true;
                   send_interested c;
                   c.client_new_chunks <- n :: c.client_new_chunks;
                   update_client_bitmap c;
-                end;
+	      | VB.State_complete | VB.State_verified -> ()
 
 (*        begin
           match c.client_bitmap, c.client_uploader with
@@ -994,7 +1002,7 @@ and client_to_client c sock msg =
                   if !verbose_msg_clients then lprintf_nl "%s:%d with software %s : Choke send, but no client bitmap"
                     (Ip.to_string ip) port (brand_to_string c.client_brand)
             | Some up ->
-                CommonSwarming.clear_uploader_ranges up
+                CommonSwarming.clear_uploader_intervals up
           end;
           c.client_ranges_sent <- [];
           c.client_range_waiting <- None;
@@ -1331,7 +1339,7 @@ let get_sources_from_tracker file =
 		t.tracker_enabled <- false;
 		(* remove_tracker t.tracker_url file; *)
                 lprintf_nl "Failure from Tracker %s in file: %s Reason: %s\nBT: Tracker %s disabled for failure"
-                  t.tracker_url file.file_name failure t.tracker_url
+                  t.tracker_url file.file_name (Charset.to_utf8 failure) t.tracker_url
             | String "warning message", String warning ->
                 lprintf_nl "Warning from Tracker %s in file: %s Reason: %s" t.tracker_url file.file_name warning
             | String "interval", Int n ->
