@@ -154,9 +154,9 @@ let check_if_interesting file c =
       (match c.client_block with
           None -> true
         | Some b ->
-            let block_num = CommonSwarming.block_num swarmer b in
-            let bitmap = CommonSwarming.verified_bitmap swarmer in
-            bitmap.[block_num] <> '3')
+            let chunk_num = CommonSwarming.block_chunk_num swarmer b in
+            let bitmap = CommonSwarming.chunks_verified_bitmap swarmer in
+            VB.get bitmap chunk_num <> VB.State_verified)
     in
     if must_send then
       begin
@@ -261,8 +261,7 @@ let new_file file_id t torrent_diskname file_temp file_state =
       else
         set_trackers file [t.torrent_announce];
       if file_state <> FileShared then begin
-          let kernel = CommonSwarming.create_swarmer file_temp (file_size file)
-            (min max_range_len file.file_piece_size) in
+          let kernel = CommonSwarming.create_swarmer file_temp (file_size file) in
           let swarmer = CommonSwarming.create kernel (as_file file)
             file.file_piece_size in
           file.file_swarmer <- Some swarmer;
