@@ -50,6 +50,8 @@ open Gettext
 let _s x = _s "BTInteractive" x
 let _b x = _b "BTInteractive" x
 
+module VB = VerificationBitmap
+
 exception Already_exists
 
 let op_file_all_sources file =
@@ -256,14 +258,24 @@ let op_file_print_html file buf =
 	  end in
   print_first_tracker file.file_trackers;
 
+  let check_magic file =
+    match Magic.M.magic_fileinfo file false with
+      None -> None
+    | Some s -> Some (HashMagic.merge CommonGlobals.files_magic s)
+  in
   let cntr = ref 0 in
-  List.iter (fun (filename, size, magic) ->
+  List.iter (fun (filename, size, _) ->
     Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
     let fs = Printf.sprintf "File %d" !cntr in
     let magic_string =
-      match magic with
-        None -> ""
-      | Some magic -> Printf.sprintf " / %s" magic
+      if !Autoconf.magic_works then
+	begin
+          let subfile = Filename.concat (file_disk_name file) filename in
+          match check_magic subfile with
+            None -> ""
+          | Some magic -> Printf.sprintf " / %s" magic
+	end
+      else ""
     in
     html_mods_td buf [
       (fs, "sr br", fs);
@@ -278,7 +290,7 @@ let op_file_print_sources_html file buf =
 
       let chunks = (match file.file_swarmer with
             None -> "" | Some swarmer ->
-              CommonSwarming.verified_bitmap swarmer) in
+              VB.to_string (CommonSwarming.chunks_verified_bitmap swarmer)) in
 
       let header_list = [
         ( "1", "srh br ac", "Client number", "Num" ) ;
@@ -310,8 +322,13 @@ let op_file_print_sources_html file buf =
         (fun i -> ((int_of_char info.G.file_chunks.[i])-48)))) ) ;
 *)
         ( "1", "srh ar", "Number of full chunks", (Printf.sprintf "%d"
-              (String.length (String2.replace
-                  (String2.replace chunks '0' "") '1' "")) )) 
+          (match file.file_swarmer with
+	  | None -> 0
+	  | Some swarmer ->
+	      let bitmap = 
+		CommonSwarming.chunks_verified_bitmap swarmer in
+	      VB.fold_lefti (fun acc _ s ->
+		if s = VB.State_verified then acc + 1 else acc)	0 bitmap)))
       ] in
 
       html_mods_table_header buf "sourcesTable" "sources al" header_list;
@@ -371,7 +388,7 @@ let op_file_check file =
     None ->
       lprintf_nl "verify_chunks: no swarmer to verify chunks"
   | Some swarmer ->
-      CommonSwarming.verify_all_chunks swarmer true
+      CommonSwarming.verify_all_chunks_immediately swarmer
 
 let remove_all_clients file =
   Hashtbl.clear file.file_clients; 
@@ -406,12 +423,12 @@ let op_file_info file =
     P.file_network = network.network_num;
     P.file_names = [file.file_name, P.noips()];
     P.file_chunks = (match file.file_swarmer with
-        None -> "" | Some swarmer ->
-          CommonSwarming.verified_bitmap swarmer);
+    | None -> None 
+    | Some swarmer -> Some (CommonSwarming.chunks_verified_bitmap swarmer));
     P.file_availability =
     [network.network_num,(match file.file_swarmer with
           None -> "" | Some swarmer ->
-            CommonSwarming.availability swarmer)];
+            CommonSwarming.chunks_availability swarmer)];
 
     P.file_chunks_age = last_seen;
     P.file_uids = [Uid.create (BTUrl file.file_id)];
@@ -440,7 +457,7 @@ let op_ft_info ft =
     P.file_state = ft_state ft;
     P.file_sources = None;
     P.file_download_rate = 0.;
-    P.file_chunks = "";
+    P.file_chunks = None;
     P.file_availability =  [network.network_num, ""];
     P.file_format = FormatNotComputed 0;
     P.file_chunks_age = [| last_time () |];
@@ -623,32 +640,6 @@ let retry_all_ft () =
           lprintf_nl "ft_retry: exception %s" (Printexc2.to_string e)
   ) ft_by_num
 
-let file_magic_check () =
-  if !Autoconf.magic_works then begin
-    if !verbose then lprintf_nl "computing sub_file magic values";
-    let check_magic file = 
-      match Magic.M.magic_fileinfo file false with
-        None -> None
-      | Some s -> Some (HashMagic.merge CommonGlobals.files_magic s)
-    in
-    Hashtbl.iter (fun _ file ->
-      let updated = ref false in
-      let new_file_files = ref [] in
-      List.iter (fun (filename, size, magic) ->
-        let subfile = Filename.concat (file_disk_name file) filename in
-        let new_magic =
-          match magic with
-            None -> check_magic subfile
-          | Some magic when magic = "data" || magic = "empty" -> check_magic subfile
-          | _ -> magic
-        in
-        if new_magic <> magic then updated := true;
-        new_file_files := (filename, size, new_magic) :: !new_file_files
-        ) file.file_files;
-        if !updated then file.file_files <- !new_file_files
-    ) files_by_uid
-  end
-
 let load_torrent_from_web r ft =
   if !verbose then
       lprintf_nl "Loading torrent from web";
