@@ -514,6 +514,7 @@ let parse_servers_list s =
           s.server_name <- server_name;
           s.server_info <- server_info;
           (try s.server_nusers <- Int64.of_string server_nusers with _ -> ());
+	  server_must_update s
       | _ -> 
           lprintf "Bad line [%s]" s; lprint_newline ();      
   ) lines;
@@ -530,21 +531,15 @@ module P = GuiTypes
   
 let _ =
   server_ops.op_server_info <- (fun s ->
-        {
+        { (impl_server_info s.server_server) with
           P.server_num = (server_num s);
           P.server_network = network.network_num;
           P.server_addr = s.server_addr;
           P.server_port = s.server_port;
-	  P.server_realport = 0;
-          P.server_score = 0;
-          P.server_tags = [];
           P.server_nusers = s.server_nusers;
-          P.server_nfiles = Int64.zero;
           P.server_state = server_state s;
           P.server_name = s.server_name;
           P.server_description = s.server_info;
-          P.server_users = None;
-          P.server_banner = "";
           P.server_preferred = false;
         }
 )
