@@ -293,7 +293,7 @@ let value_to_file file_size file_state assocs =
   (match file.file_swarmer with
       None -> ()
     | Some swarmer ->
-        CommonSwarming.value_to_swarmer swarmer assocs;
+        CommonSwarming.value_to_frontend swarmer assocs;
         CommonSwarming.set_verifier swarmer (if md4s = [||] then
             VerificationNotAvailable
           else
@@ -317,7 +317,7 @@ let file_to_value file =
   match file.file_swarmer with
     None -> fields
     | Some swarmer ->
-        CommonSwarming.swarmer_to_value swarmer fields
+        CommonSwarming.frontend_to_value swarmer fields
   in
   fields
     
