@@ -55,6 +55,7 @@ open CommonOptions
 open DonkeyStats
 open DonkeyUdp
 
+module VB = VerificationBitmap
 
 let result_name r =
   match r.result_names with
@@ -492,7 +493,7 @@ let recover_md4s md4 =
   match file.file_swarmer with
     None -> ()
   | Some swarmer ->
-      CommonSwarming.verify_all_chunks swarmer false
+      CommonSwarming.verify_all_chunks swarmer
 
   (*
   if file.file_chunks <> [||] then
@@ -598,7 +599,7 @@ let op_file_check file =
   match file.file_swarmer with
     None -> ()
   | Some swarmer ->
-      CommonSwarming.verify_all_chunks swarmer true
+      CommonSwarming.verify_all_chunks_immediately swarmer
 
 let register_commands list =
   register_commands
@@ -1027,15 +1028,15 @@ let _ =
             P.file_all_sources = file.file_sources.DonkeySources.manager_all_sources;
             P.file_active_sources = file.file_sources.DonkeySources.manager_active_sources;
             P.file_chunks = 
-                  (match file.file_swarmer with
-                   | None -> "" 
-                   | Some swarmer -> CommonSwarming.verified_bitmap swarmer);
+                (match file.file_swarmer with
+                | None -> None
+                | Some swarmer -> Some (CommonSwarming.chunks_verified_bitmap swarmer));
             P.file_availability =
               [
                 network.network_num,
                 (match file.file_swarmer with
                   | None -> "" 
-                  | Some swarmer -> CommonSwarming.availability swarmer)
+                  | Some swarmer -> CommonSwarming.chunks_availability swarmer)
               ];
             P.file_format = file.file_format;
             P.file_chunks_age = last_seen;
@@ -1292,8 +1293,8 @@ parent.fstatus.location.href='submit?q=rename+%d+\\\"'+encodeURIComponent(formID
 
       let chunks =
       (match file.file_swarmer with
-       None -> "" | Some swarmer ->
-       CommonSwarming.verified_bitmap swarmer)
+      | None -> None 
+      | Some swarmer -> Some (CommonSwarming.chunks_verified_bitmap swarmer))
       in
 
       html_mods_table_header buf "sourcesTable" "sources al" ([
@@ -1326,12 +1327,16 @@ parent.fstatus.location.href='submit?q=rename+%d+\\\"'+encodeURIComponent(formID
           ( "1", "srh ar br", "Connected time (minutes)", "CT" ) ;
           ( "0", "srh br", "Client MD4", "MD4" ) ;
           ( "0", "srh", "Chunks (absent|partial|present|verified)",
-            (colored_chunks
-                (Array.init (String.length chunks)
-                (fun i -> ((int_of_char chunks.[i])-48)))) ) ;
+	  match chunks with
+	  | None -> ""
+	  | Some chunks -> colored_chunks chunks) ;
           ( "1", "srh ar", "Number of full chunks", (Printf.sprintf "%d"
-                (let fc = ref 0 in (String.iter (fun s -> if s = '3' then incr fc)
-                  chunks );!fc ))) ]);
+            (match chunks with
+	    | None -> 0
+	    | Some chunks ->
+		VerificationBitmap.fold_lefti (fun acc _ s ->
+		  if s = VerificationBitmap.State_verified then acc + 1
+		  else acc) 0 chunks))) ]);
 
 
       html_mods_cntr_init();
@@ -1409,8 +1414,12 @@ parent.fstatus.location.href='submit?q=rename+%d+\\\"'+encodeURIComponent(formID
                   let _, qchunks,_ = List.find (fun (qfile, _,_) ->
                         qfile == file) qfiles in
                   let tc = ref 0 in
-                  let arr = Array.init (Bitv.length qchunks) 
-                              (fun i -> if Bitv.get qchunks i then begin incr tc; 2 end else 0) in
+		  let arr = 
+		    VerificationBitmap.init (Bitv.length qchunks) (fun i -> 
+		    if Bitv.get qchunks i then begin
+		      incr tc;
+		      VerificationBitmap.State_complete
+		    end else VerificationBitmap.State_missing) in
                   Printf.bprintf buf "%s\\</td\\>\\<td class=\\\"sr ar\\\"\\>%d\\</td\\>"
                     (CommonFile.colored_chunks arr) !tc;
                 with Not_found -> (
@@ -1541,23 +1550,16 @@ let _ =
       | Some files ->
           List2.tail_map (fun r -> "", r) files);
   client_ops.op_client_browse <- (fun c immediate ->
-      lprintf_nl "*************** should browse  ***********";
+      if !verbose then lprintf_nl "connecting friend %s" (full_client_identifier c);
       match c.client_source.DonkeySources.source_sock with
-      | Connection sock    ->
-(*
-      lprintf "****************************************";
-      lprint_newline ();
-      lprintf "       ASK VIEW FILES         ";
-lprint_newline ();
-  *)
+      | Connection sock ->
+	  if !verbose then lprintf_nl "retrieving filelist from friend %s" (full_client_identifier c);
           client_send c (
             let module M = DonkeyProtoClient in
             let module C = M.ViewFiles in
             M.ViewFilesReq C.t);
       | NoConnection ->
-          lprintf_nl "****************************************";
-          lprintf_nl "       TRYING TO CONTACT FRIEND";
-
+	  if !verbose then lprintf_nl "re-connecting friend %s" (full_client_identifier c);
           reconnect_client c
       | _ -> ()
   );
