@@ -133,9 +133,6 @@ let declare_completed_file file =
   CommonSwarming.remove_swarmer file.file_swarmer;
   file.file_swarmer <- None;
   unshare_file file;
-  ignore (CommonShared.new_shared "completed" 0 (
-      file_best_name file )
-    (file_disk_name file));
   (try
       let format = CommonMultimedia.get_info
           (file_disk_name file) in
@@ -162,14 +159,9 @@ let check_file_downloaded file =
       match file.file_swarmer with
         None -> ()
       | Some swarmer ->
-          let bitmap = CommonSwarming.verified_bitmap swarmer in
+          let bitmap = CommonSwarming.chunks_verified_bitmap swarmer in
 (*          lprintf "Verified bitmap: [%s]\n" bitmap; *)
-          let rec iter i =
-            if i =  String.length bitmap then true
-            else
-            if bitmap.[i] = '3' then iter (i+1) else false
-          in
-          let verified = iter 0 in
+	  let verified = VB.for_all (( = ) VB.State_verified) bitmap in
           
           let downloaded = CommonSwarming.downloaded swarmer in
           if file_downloaded file <> downloaded then begin
@@ -193,15 +185,9 @@ let check_files_downloaded () =
           match file.file_swarmer with
             None -> ()
           | Some swarmer ->
-              let bitmap = CommonSwarming.verified_bitmap swarmer in
-              let rec iter i len =
-                if i < len then
-                  if bitmap.[i] = '3' then
-                    DonkeyShare.must_share_file file
-                  else
-                    iter (i+1) len
-              in
-              iter 0 (String.length bitmap)
+              let bitmap = CommonSwarming.chunks_verified_bitmap swarmer in
+	      if VB.existsi (fun _ s -> s = VB.State_verified) bitmap then
+                DonkeyShare.must_share_file file
   ) files_by_md4
 
 let add_client_chunks c file client_chunks =
@@ -213,7 +199,7 @@ let add_client_chunks c file client_chunks =
           (f, chunks, up) :: tail ->
             if f != file then iter tail
             else begin
-                CommonSwarming.update_uploader up
+                CommonSwarming.update_uploader_intervals up
                 (AvailableBitv client_chunks);
                 Bitv.blit client_chunks 0 chunks 0 (Bitv.length chunks)
               end
@@ -235,8 +221,7 @@ let clean_current_download c =
   match c.client_download with
     None -> ()
   | Some (file, up) ->
-      CommonSwarming.clear_uploader_block up;
-      CommonSwarming.clear_uploader_ranges up;
+      CommonSwarming.unregister_uploader up;
       c.client_download <- None
 
 let send_get_range_request c file ranges = 
@@ -329,7 +314,7 @@ let rec get_from_client c =
           let rec iter n =
             if n < 3 then
               try
-                ignore (CommonSwarming.find_range up);
+                ignore (CommonSwarming.find_range up zone_size);
                 iter (n+1)
               with 
                 Not_found -> n
@@ -446,7 +431,8 @@ let block_received c md4 begin_pos bloc bloc_pos bloc_len =
 (*                    Printf2.lprint_string m; *)
                 CommonEvent.add_event (Console_message_event m);
                 if e <> End_of_file then begin
-                    let m = "File paused.\n" in
+                    let m = Printf.sprintf "File %s paused, exception %s.\n"
+			      (file_best_name file) (Printexc2.to_string e) in
                     Printf2.lprint_string m;
                     CommonEvent.add_event (Console_message_event m);
                     file_pause (as_file file);
