@@ -58,9 +58,6 @@ let max_udp_sends = define_expert_option donkey_section ["max_udp_sends"]
 let reliable_sources = define_option donkey_section ["reliable_sources"]
     "Should mldonkey try to detect sources responsible for corruption and ban them" bool_option true
 
-let emule_compression = define_option donkey_section ["emule_compression"]
-  "Should mldonkey accept compressed packets from emule" bool_option true
-
 let ban_identity_thieves = define_option donkey_section ["ban_identity_thieves"]
   "Should mldonkey try to detect sources masquerading as others and ban them" bool_option true
 
@@ -110,7 +107,8 @@ let keep_best_server = define_expert_option donkey_section
 
 let connect_only_preferred_server = define_expert_option donkey_section
   ["connect_only_preferred_server"]
-  "only servers which are set to 'preferred' will be connected"
+  "only servers which are set to 'preferred' will be connected, 
+  please note that any server must be set to preferred to use this feature"
   bool_option false
 
 let max_walker_servers = define_expert_option donkey_section
@@ -216,7 +214,8 @@ let upload_power = define_expert_option donkey_section ["upload_power"]
 let max_server_age = define_expert_option donkey_section ["max_server_age"] "max number of days after which an unconnected server is removed" int_option 2
 
 let remove_old_servers_delay = define_expert_option donkey_section ["remove_old_servers_delay"]
-  "How often should remove old donkey servers (see max_server_age) be called (in seconds, 0 to disable)"
+  "How often should remove old donkey servers (see max_server_age) be called
+   (in seconds, 0 to disable)"
     float_option 900.
 
 let min_left_servers = define_expert_option donkey_section ["min_left_servers"]
