@@ -305,7 +305,7 @@ let new_file file_temporary file_name file_size file_hash =
     }
   in
   incr search_num;
-  let kernel = CommonSwarming.create_swarmer file_temp file_size min_range_size in
+  let kernel = CommonSwarming.create_swarmer file_temp file_size in
   let swarmer = CommonSwarming.create kernel (as_file file)
       file_chunk_size in
   file.file_swarmer <- Some swarmer;
