@@ -47,6 +47,8 @@ open Gettext
 let _s x = _s "FileTPInteractive" x
 let _b x = _b "FileTPInteractive" x
 
+module VB = VerificationBitmap
+
 let _ =
   network.op_network_connected <- (fun _ -> true);
   network.op_network_connected_servers <- (fun _ -> [])
@@ -66,10 +68,22 @@ let _ =
 
 module P = GuiTypes
 
-let _ =
-  file_ops.op_file_cancel <- (fun file ->
+
+let clean_stop file =
       CommonSwarming.remove_swarmer file.file_swarmer;
       file.file_swarmer <- None;
+  List.iter (fun c ->
+    c.client_downloads <- List.filter 
+      (fun cd -> cd.download_file.file_id <> file.file_id) c.client_downloads;
+    if (List.length c.client_downloads) == 0 then begin
+      FileTPClients.disconnect_client c Closed_by_user;
+      client_remove c;
+    end;
+  ) file.file_clients
+
+let _ =
+  file_ops.op_file_cancel <- (fun file ->
+      clean_stop file;
       remove_file file;
   );
   file_ops.op_file_info <- (fun file ->
@@ -80,12 +94,13 @@ let _ =
         P.file_all_sources = (List.length file.file_clients);
         P.file_active_sources = (List.length file.file_clients);
         P.file_chunks = (match file.file_swarmer with
-          None -> "" | Some swarmer ->
-            CommonSwarming.verified_bitmap swarmer);
+        | None -> None 
+	| Some swarmer ->
+            Some (CommonSwarming.chunks_verified_bitmap swarmer));
         P.file_availability =
         [network.network_num,(match file.file_swarmer with
           None -> "" | Some swarmer ->
-            CommonSwarming.availability swarmer)];
+            CommonSwarming.chunks_availability swarmer)];
         P.file_format = FormatNotComputed 0;
         P.file_uids = [Uid.create (FileTP file.file_id)];
       }
@@ -107,7 +122,7 @@ let _ =
           c.client_hostname c.client_port);
         P.client_num = (client_num (as_client c));
         P.client_connect_time = BasicSocket.last_time ();
-        P.client_software = "TP";
+        P.client_software = c.client_software;
         P.client_downloaded = c.client_downloaded;
       }
   );
@@ -394,11 +409,17 @@ let _ =
   network.op_network_share <- (fun fullname codedname size -> ());
   network.op_network_search <- (fun ss buf -> ());
   network.op_network_download <- (fun r -> dummy_file);
-  file_ops.op_file_commit <- (fun file new_name -> ());
-  file_ops.op_file_pause <- (fun file -> ());
+  file_ops.op_file_commit <- (fun file new_name -> clean_stop file);
+  file_ops.op_file_pause <- (fun file -> 
+    List.iter (fun c ->
+      match c.client_connected_for with
+      | Some s when s.file_id = file.file_id -> 
+          FileTPClients.disconnect_client c Closed_by_user;
+      | _ -> ()
+    ) file.file_clients
+  );
   file_ops.op_file_resume <- (fun file -> ());
   file_ops.op_file_print_html <- (fun file buf -> ());
-  file_ops.op_file_print_sources_html <- (fun file buf -> ());
   network.op_network_forget_search <- (fun s -> ());
   network.op_network_connect_servers <- (fun s -> ());
   network.op_network_recover_temp <- (fun s -> ())
