@@ -47,6 +47,8 @@ open GnutellaGlobals
 open GnutellaComplexOptions
 open GnutellaProtocol
 
+module VB = VerificationBitmap
+
 (* Don't share files greater than 10 MB on Gnutella and limit to 200 files. 
  Why ? Because we don't store URNs currently, and we don't want mldonkey
  to compute hashes for hours at startup *)
@@ -262,7 +264,7 @@ let _ =
           match file.file_ttr with
             None -> failwith "No TTR for verification"
           | Some ttt ->
-              CommonSwarming.verify_all_chunks swarmer true  
+              CommonSwarming.verify_all_chunks_immediately swarmer
   );  
   
   file_ops.op_file_recover <- (fun file ->
@@ -293,17 +295,17 @@ let _ =
   );
   file_ops.op_file_info <- (fun file ->
       { (impl_file_info file.file_file) with
-        P.file_name = file.file_name;
         P.file_network = network.network_num;
         P.file_names = [file.file_name, P.noips()];
         
          P.file_chunks = (match file.file_swarmer with
-             None -> "" | Some swarmer ->
-               CommonSwarming.verified_bitmap swarmer);
+         | None -> None 
+	 | Some swarmer -> 
+	     Some (CommonSwarming.chunks_verified_bitmap swarmer));
         P.file_availability =   [network.network_num,
            (match file.file_swarmer with
            None -> "" | Some swarmer ->
-                 CommonSwarming.availability swarmer)];
+                 CommonSwarming.chunks_availability swarmer)];
         
         P.file_chunks_age = [|0|];
         P.file_last_seen = BasicSocket.last_time ();
@@ -402,10 +404,6 @@ let _ =
           else "", false
   )
   
-let browse_client c = 
-  lprintf "Gnutella: browse client not implemented\n";
-  ()
-  
 let _ =
   client_ops.op_client_info <- (fun c ->
       { (impl_client_info c.client_client) with
@@ -420,8 +418,7 @@ let _ =
 
       }
   );
-  client_ops.op_client_browse <- (fun c immediate ->
-      browse_client c
+  client_ops.op_client_browse <- (fun _ _ -> lprintf_nl "Gnutella: browse client not implemented";
   );
 
     client_ops.op_client_bprint <- (fun c buf ->
