@@ -3,14 +3,14 @@ open Printf2
 open Ip
 
 (* prints a new logline with date, module and starts newline *)
-let lprintf_nl () =
-  lprintf "%s[IPblock] "
-  (log_time ()); lprintf_nl2
+let log_prefix = "[IPblock]"
+
+let lprintf_nl fmt =
+  lprintf_nl2 log_prefix fmt
 
 (* prints a new logline with date, module and does not start newline *)
-let lprintf_n () =
-  lprintf "%s[IPblock] "
-  (log_time ()); lprintf
+let lprintf_n fmt =
+  lprintf2 log_prefix fmt
 
 module H = Weak.Make(struct
 	type t = string
@@ -19,8 +19,6 @@ module H = Weak.Make(struct
       end)
 
 let descriptions = H.create 13
-let ranges_1 = ref 0 (* ranges in blocklist file *)
-let ranges_2 = ref 0 (* ranges after optimization *)
 
 let shared_description s =
   (* Currently trims strings left and right;
@@ -112,8 +110,7 @@ let rec match_ip_aux bl ip =
 	  Some br
 
 let match_ip bl ip =
-  let m = if Ip.local_ip ip then None else
-  match_ip_aux bl ip in
+  let m = match_ip_aux bl ip in
   (match m with
       Some br ->
 	br.blocking_hits <- br.blocking_hits + 1
@@ -171,11 +168,11 @@ let load_merge bl filename remove =
   let guardian_regexp = Str.regexp "^\\(.*\\): *\\([0-9]+\\.[0-9]+\\.[0-9]+\\.[0-9]+\\)-\\([0-9]+\\.[0-9]+\\.[0-9]+\\.[0-9]+\\)" in
   let ipfilter_regexp = Str.regexp "^\\([0-9]+\\.[0-9]+\\.[0-9]+\\.[0-9]+\\) *- *\\([0-9]+\\.[0-9]+\\.[0-9]+\\.[0-9]+\\) *, *[0-9]+ *, *\\(.*\\)$" in
 
-  let cin = open_in filename in
   let bl = ref bl in
   let nranges = ref 0 in
-  let nlines = ref 0 in
   let error = ref false in
+  Unix2.tryopen_read filename (fun cin ->
+  let nlines = ref 0 in
   try
     while true do
       let line = input_line cin in
@@ -209,20 +206,16 @@ let load_merge bl filename remove =
 	with _ ->
 	  if not !error then
 	    begin
-	      lprintf_n () "Syntax error while loading IP blocklist in line";
+        lprintf_n "Syntax error while loading IP blocklist in line";
 	      error := true
 	    end;
 	    lprintf " %d" !nlines;
-    done;
-    bl_empty (* not reached *)
-  with End_of_file ->
-    if !error then lprint_newline ();
-    close_in cin;
-    if remove then (try Sys.remove filename with _ -> ());
-    let optimized_bl = bl_optimize !bl in
-    lprintf_nl () "%d ranges loaded - optimized to %d" !nranges (bl_length optimized_bl);
-    ranges_1 := !nranges;
-    ranges_2 := bl_length optimized_bl;
+    done
+  with End_of_file -> ());
+  if !error then lprint_newline ();
+  if remove then (try Sys.remove filename with _ -> ());
+  let optimized_bl = bl_optimize !bl in
+  lprintf_nl "%d ranges loaded - optimized to %d" !nranges (bl_length optimized_bl);
 (*    bl_optimizedp optimized_bl;
     for i=0 to 999999 do
       let random_ip = Ip.of_ints (Random.int 256, Random.int 256, Random.int 256, Random.int 256) in
@@ -231,70 +224,66 @@ let load_merge bl filename remove =
 	| Some _, Some _ -> ()
 	| _ -> assert false
     done; *)
-    optimized_bl
+  optimized_bl
 
 let load filename =
-   lprintf_nl () "loading %s" filename;
-   if Sys.file_exists filename then
-   let ext = String.lowercase (Filename2.extension filename) in
+  lprintf_nl "loading %s" filename;
+  if Sys.file_exists filename then
     let last_ext = String.lowercase (Filename2.last_extension filename) in
-    let real_ext = if last_ext = ".zip" then
-      last_ext
-    else
-      ext
-    in
-      match real_ext with
-        ".zip" ->
-	   begin
-	   try
-	     let ic = Zip.open_in filename in
-	     try
-	       let file = Zip.find_entry ic "guarding.p2p" in
-	         Zip.close_in ic;
-		 lprintf_nl () "guarding.p2p found in zip file";
-		 let _ = Misc.archive_extract filename "zip" in
-		 load_merge bl_empty file.Zip.filename true
-	       with e ->
-		 begin
-		   try
-		     let file = Zip.find_entry ic "guarding_full.p2p" in
-		       Zip.close_in ic;
-		       lprintf_nl () "guarding_full.p2p found in zip file";
-		       let _ = Misc.archive_extract filename "zip" in
-		       load_merge bl_empty file.Zip.filename true
-		     with e ->
-		       Zip.close_in ic;
-		       lprintf_nl () "Exception %s while extracting guarding.p2p/guarding_full.p2p from %s"
-			 (Printexc2.to_string e) filename;
-		       lprintf_nl () "One of the mentioned files has to be present in the zip file";
-		       bl_empty
-		 end
-	     with e ->
-	       lprintf_nl () "Exception %s while opening %s"
-		 (Printexc2.to_string e) filename;
-	       bl_empty
-	   end
-       | ".bz2" | ".p2p.bz2" | ".gz" | ".p2p.gz" ->
-	   begin
-	     let filetype =
-	       if ext = ".bz2" || ext = ".p2p.bz2" then "bz2" else "gz" in
-	     try
-		 let s = Misc.archive_extract filename filetype in
-		   load_merge bl_empty s true
-	       with e ->
-		 lprintf_nl () "Exception %s while extracting from %s"
-		   (Printexc2.to_string e) filename;
-		   bl_empty
-	   end
-       | ".tar.bz2" | ".p2p.tar.bz2" | ".tar.gz" | ".p2p.tar.gz" ->
-	    lprintf_nl () "tar files are not (yet) supported, please untar %s" filename;
+    if last_ext = ".zip" then
+      let filenames_list = 
+	["guarding.p2p"; "guarding_full.p2p"; "ipfilter.dat"] in
+      (try
+	Unix2.tryopen_read_zip filename (fun ic ->
+	  try
+	    let rec find_in_zip l =
+	      match l with
+		| [] -> raise Not_found
+		| h :: q ->
+		    try
+		      let file = Zip.find_entry ic h in
+          lprintf_nl "%s found in zip file" h;
+		      ignore(Misc.archive_extract filename "zip");
+		      load_merge bl_empty file.Zip.filename true
+		    with Not_found ->
+		      find_in_zip q in
+	    find_in_zip filenames_list
+	  with e ->
+      lprintf_nl "Exception %s while extracting %s from %s"
+	      (Printexc2.to_string e) 
+	      (String.concat "/" filenames_list)
+	      filename;
+      lprintf_nl "One of the mentioned files has to be present in the zip file";
+	    bl_empty)
+      with e ->
+  lprintf_nl "Exception %s while opening %s"
+	  (Printexc2.to_string e)
+	  filename;
+	bl_empty)
+    else     
+      let ext = String.lowercase (Filename2.extension filename) in
+      match ext with
+	| ".bz2" | ".p2p.bz2" | ".dat.bz2" 
+	| ".gz"  | ".p2p.gz"  | ".dat.gz" ->
+	    let filetype =
+	      if String2.check_suffix ext ".bz2" then "bz2" else "gz" in
+	    (try
+	      let s = Misc.archive_extract filename filetype in
+	      load_merge bl_empty s true
+	    with e ->
+        lprintf_nl "Exception %s while extracting from %s"
+		(Printexc2.to_string e) filename;
+	      bl_empty)
+        | ".tar.bz2" | ".p2p.tar.bz2" | ".dat.tar.bz2"
+        | ".tar.gz" | ".p2p.tar.gz" | ".dat.tar.gz" ->
+      lprintf_nl "tar files are not (yet) supported, please untar %s" filename;
 	    bl_empty
-       | _ -> load_merge bl_empty filename false
-   else
-     begin
-       lprintf_nl () "file %s not found" filename;
-       bl_empty
-     end
+        | _ -> load_merge bl_empty filename false
+  else
+    begin
+      lprintf_nl "file %s not found" filename;
+      bl_empty
+    end
 
 let of_list l =
   let rec of_list_aux l bl =
@@ -346,20 +335,8 @@ let print_list buf bl =
           let nright = print_list_aux right in
           nleft + 1 + nright in
 
-  ignore(print_list_aux bl);
-  if !ranges_1 = 0 then
-    Printf.bprintf buf "%d ranges loaded\n" !ranges_1
-  else
-    Printf.bprintf buf "%d ranges loaded - optimized to %d\n" !ranges_1 !ranges_2
-
-let bl = ref BL_Empty
-
-let ip_blocked ip =
-  match match_ip !bl ip with
-    None -> false
-  | Some br -> true
-
-let block_stats () = (!ranges_1, !ranges_2)
+  let nranges = print_list_aux bl in
+  Printf.bprintf buf "%d ranges\n" nranges
 
 (*
 open Benchmark
@@ -386,7 +363,4 @@ let _ =
   Heap.add_memstat "Ip_set" (fun level buf ->
       let counter = ref 0 in
       H.iter (fun _ -> incr counter) descriptions;
-      Printf.bprintf buf "  descriptions: %d\n" !counter;
-      Printf.bprintf buf "  ranges: %d\n" (bl_length !bl);  
-  )
-
+      Printf.bprintf buf "  descriptions: %d\n" !counter)
